/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - abstract-home.routable.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2023 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  Directive,
  Injector,
  OnInit,
} from "@angular/core";
import {TranslateService} from "@ngx-translate/core";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {SOLIDIFY_CONSTANTS} from "../../../constants";
import {DataTestPartialEnum} from "../../../enums/partial/data-test-partial.enum";
import {RoutesPartialEnum} from "../../../enums/partial/routes-partial.enum";
import {ExtendEnum} from "../../../types/extend-enum.type";
import {AbstractInternalRoutable} from "../abstract-internal/abstract-internal.routable";

@Directive()
// eslint-disable-next-line @angular-eslint/directive-class-suffix
export abstract class AbstractHomeRoutable extends AbstractInternalRoutable implements OnInit {
  searchValueInUrl: string = SOLIDIFY_CONSTANTS.STRING_EMPTY;

  protected abstract _tiles: HomeTileModel[];

  protected constructor(protected readonly _store: Store,
                        protected readonly _translate: TranslateService,
                        protected readonly _injector: Injector) {
    super(_injector);
  }

  navigate(path: ExtendEnum<RoutesPartialEnum>): void {
    this._store.dispatch(new Navigate([path]));
  }

  ngOnInit(): void {
    this._computeTilesToDisplay();
  }

  visibleTiles: HomeTileModel[];

  private _getVisibleTiles(): HomeTileModel[] {
    return this._tiles.filter(tile => this._displayTile(tile));
  }

  protected abstract _displayTile(tile: HomeTileModel): boolean;

  search(searchTerm: string): void {
    this.searchValueInUrl = searchTerm.toLowerCase();
    this._computeTilesToDisplay();
    return;
  }

  private _computeTilesToDisplay(): void {
    this.visibleTiles = this._getVisibleTiles();
  }
}

export interface HomeTileModel {
  avatarIcon: string;
  titleToTranslate: string;
  subtitleToTranslate?: string;
  path: ExtendEnum<RoutesPartialEnum>;
  permission: string;
  dataTest?: ExtendEnum<DataTestPartialEnum>;
}
